@@ -1,5 +1,5 @@
 package Crypt::Argon2;
-$Crypt::Argon2::VERSION = '0.007';
+$Crypt::Argon2::VERSION = '0.008';
 use strict;
 use warnings;
 
@@ -7,10 +7,33 @@ use Exporter 5.57 'import';
 our @EXPORT_OK = qw/
 	argon2id_raw argon2id_pass argon2id_verify
 	argon2i_raw argon2i_pass argon2i_verify
-	argon2d_raw/;
+	argon2d_raw argon2_needs_rehash/;
 use XSLoader;
 XSLoader::load(__PACKAGE__, __PACKAGE__->VERSION || 0);
 
+use MIME::Base64 'decode_base64';
+
+my %multiplier = (
+	k => 1,
+	M => 1024,
+	G => 1024 * 1024,
+);
+
+sub argon2_needs_rehash {
+	my ($encoded, $type, $t_cost, $m_cost, $parallelism, $output_length, $salt_length) = @_;
+	$m_cost =~ s/ \A (\d+) ([kMG]) \z / $1 * $multiplier{$2} /xmse;
+	my (undef, $name, $version, $argstring, $salt, $hash) = split /\$/, $encoded;
+	return 1 if $name ne $type;
+	return 1 if $version !~ /v=(\d+)/ or $1 != 19;
+	my %args;
+	while ($argstring =~ m/(\w)=(\d+)/gc) {
+		$args{$1} = $2;
+	}
+	return 1 if $args{t} != $t_cost or $args{m} != $m_cost or $args{p} != $parallelism;
+	return 1 if length decode_base64($salt) != $salt_length or length decode_base64($hash) != $output_length;
+	return 0;
+}
+
 1;
 
 # ABSTRACT: Perl interface to the Argon2 key derivation functions
@@ -27,7 +50,7 @@ Crypt::Argon2 - Perl interface to the Argon2 key derivation functions
 
 =head1 VERSION
 
-version 0.007
+version 0.008
 
 =head1 SYNOPSIS
 
@@ -108,6 +131,10 @@ This function processes the C<$password> with the given C<$salt> and parameters
 
 This function processes the C<$password> with the given C<$salt> and parameters much like C<argon2i_pass>, but returns a binary tag for argon2d instead of a formatted string for argon2i.
 
+=head2 argon2_needs_rehash($encoded, $type, $t_cost, $m_cost, $parallelism, $salt_length, $output_length)
+
+This function checks if a password-encoded string needs a rehash. It will return true if the C<$type> (valid values are C<argon2i>, C<argon2id> or C<argon2d>), C<$t_cost>, C<$m_cost>, C<$parallelism>, C<$salt_length> or C<$argon2d> arguments don't match the password-encoded hash
+
 =head1 RECOMMENDED SETTINGS
 
 The following procedure to find settings can be followed:
@@ -150,6 +177,10 @@ Its RAND_bytes function is OpenSSL's pseudo-randomness source.
 
 A minimalistic abstraction around OS-provided non-blocking (pseudo-)randomness.
 
+=item * C</dev/random> / C</dev/urandom>
+
+A Linux/BSD specific pseudo-file that will allow you to read random bytes.
+
 =back
 
 Implementations of other similar algorithms include:
@@ -162,7 +193,7 @@ An implementation of scrypt, a older scheme that also tries to be memory hard.
 
 =item * L<Crypt::Eksblowfish::Bcrypt|Crypt::Eksblowfish::Bcrypt>
 
-An implementation of bcrypt, a battle-tested algortihm that tries to be CPU but not particularly memory intensive.
+An implementation of bcrypt, a battle-tested algorithm that tries to be CPU but not particularly memory intensive.
 
 =back
 
